<?php

/* Copyright (C) 2012 Stephen Larroque  <lrq3000@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * or see http://www.gnu.org/
 */

/**
 *	\file       htdocs/customfields/lib/customfields_aux.lib.php
 *	\brief      Functions to simplify the use of the CustomFields main class.
 *	\description    Here are stored functions that use the CustomFields main class but can't be implemented inside (eg: because of instancing a specific CustomFields object for a specific module, or because we need multiple CustomFields objects at once, etc.)
 *	\ingroup    customfields
 *	\version    $Id: customfields.lib.php, v1.3.1
 */

/**
 *  Fill a specified object with customfields of a specified second object (can be the same or another module's object, this allows to use customfields from other modules)
 *  @param $object        to object (object where customfields will be stored)
 *  @param $fromobject       from object (needs to at least contain 2 fields: table_element (module's name) and id (or $idvar, which contain the record's id you want to fetch)) - you can also create a dummy $fromobject with these only two fields to use this function
 *  @param $prefix          store customfields in a subobject (eg: $prefix = 'mycategory' will store customfield cf_myfield inside $object->customfields->mycategory->cf_myfield)
 *  @param $pdfformat      null/false/true      beautify the customfields values? (null = no beautify nor translation; false = beautify and translate; true = translation and pdf beautify with html entities encoding)
 *  @return     original $object populated and return a $customfields object generated with $fromobject (which allows to get the geometry of the customfields table that served to fill the $object, useful to translate label with findLabel() or findLabelPDF() )
 *
 *  Note: values are returned formatted and translated (by default normal, or PDF wise or not formatted if specified), but labels (keys) are NOT returned formatted (not translated by default because a field should always be accessible by a base name, whatever the translation is).
 */

function customfields_fill_object(&$object,$fromobject = null, $outputlangs = null, $prefix = null,$pdfformat = false) {
    global $conf, $db;

    if (!isset($fromobject)) $fromobject = $object; // by default, $fromobject is the same as $object

    // -- Include necessary files
    include_once(DOL_DOCUMENT_ROOT.'/customfields/class/customfields.class.php'); // to fetch customfields database
    include(DOL_DOCUMENT_ROOT."/customfields/conf/conf_customfields.lib.php"); // to detect the current module's parameters
    include_once(DOL_DOCUMENT_ROOT."/customfields/conf/conf_customfields_func.lib.php"); // to parse module's parameters (from config)

    // -- Find the current record's id
    // set id variable if specified in $modulesarray (by default = 'id', but can be 'socid', and other fancy stuffs)
    $idvar = 'id';
    $tmpmod = array_extract_recursive(array('table_element'=>$fromobject->table_element), $modulesarray); // Extract the subarray containing the found context
    if (isset($tmpmod[0]['idvar'])) $idvar = $tmpmod[0]['idvar'];
    // fetch the id (might be stored in an $idvar field or simply in a standard 'id' field)
    if (isset($fromobject->$idvar)) {
        $id = $fromobject->$idvar;
    } else {
        $id = $fromobject->id;
    }

    // -- Fetch customfields data
    // Loading a CustomFields object tailored for the current module
    $customfields = new CustomFields($db, $fromobject->table_element);
    // Fetch the customfields (columns names)
    $columns = $customfields->fetchAllCustomFields();
    // Fetch the current record (saved data, what we want to print in the end)
    $record = $customfields->fetch($id);

    // -- Begin to populate the substitution array with customfields data
    foreach ($columns as $field) { // One field at a time
        // Formatting the data
        $name = $customfields->varprefix.$field->column_name; // name of the property (name of the customfield column, eg: cf_user)
        //$translatedname = $customfields->findLabelPDF($field->column_name, $outputlangs); // UNUSED. Label of the customfield (as it was translated in customfields-user.lang, eg: cf_user might become 'My User')
        $value = $record->{$field->column_name}; // unformatted value (eg: cf_user = 2 = user id), we need it in order to print a beautified value (eg: ids replaced by strings) and to make the link for constraints
        if (!isset($pdfformat)) { // no formatting
            $fmvalue = $value;
        } elseif ($pdfformat) {
            $fmvalue = $customfields->printFieldPDF($field, $value, $outputlangs); // PDF formatted and translated value (cleaned and properly formatted, eg: cf_user value = 'John Doe') of the customfield
        } else {
            $fmvalue = $customfields->printField($field, $value, $outputlangs); // translated value
        }

        // Add this customfield's data to the substitution array (will automatically be replaced inside the ODT, eg: {cf_user} becomes 'John Doe')
        if ($prefix) { // prepending the prefix whether it was specified or not
            $object->customfields->$prefix->$name = $fmvalue; // adding this value to an odt variable (format: {cf_customfield} by default if varprefix is default)
        } else {
            $object->customfields->$name = $fmvalue;
        }

        //print("name: $name - trname: $translatedname - field: {$field->column_name} - value: $fmvalue\n"); // debugline

        // Constraints substitution
        // if the customfield has a constraint, we fetch all the datas from this constraint in the referenced table (so that related field to a customfield with constraints will also be substitutable)
        if (!empty($field->referenced_table_name)) {
            $fkrecord = $customfields->fetchAny('*', $field->referenced_table_name, $field->referenced_column_name."='".$value."'"); // we fetch the record in the referenced table

            if (!empty($fkrecord)) { // normally, this should never happen since a constrained customfield is always linked to a foreign record, but in case that happens, we skip to avoid errors...
                foreach ($fkrecord as $column_name => $value) { // for each foreign record, we add the value to an odt variable (format eg: base field: cf_user, constrants fields: cf_user_name, cf_user_firstname, etc..)
                    // Prepending the prefix if it was specified
                    if ($prefix) {
                        $object->customfields->$prefix->{$name.'_'.$column_name} = $value; // Saving the field's data in the object
                    } else {
                        $object->customfields->{$name.'_'.$column_name} = $value;
                    }
                }
            }
        }
    }

    return $customfields;
}

?>